local assets =
{
    Asset("ANIM", "anim/cave_ferns_potted.zip"),
    Asset("ANIM", "anim/berry.zip"),
}

local prefabs =
{
    "collapse_small",
}

local function playanim( inst, anim )
    inst.AnimState:OverrideSymbol("ferns_potted01", "berry", inst.animbuild .. anim)
end

local function makeemptyfn(inst)
    if inst:HasTag("withered") then
        playanim( inst, "0")
    else
        playanim( inst, "0")
    end
end

local function makebarrenfn(inst, wasempty)
    if inst:HasTag("withered") then
        playanim( inst, "4")
    else
        playanim( inst, "4")
    end
end

local function pickanim(inst)
    if inst.components.pickable == nil then
        return "0"
    elseif not inst.components.pickable:CanBePicked() then
        return inst.components.pickable:IsBarren() and "4" or "0"
    end

    --V2C: nil cycles_left means unlimited picks, so use max value for math
    local percent = inst.components.pickable.cycles_left ~= nil and inst.components.pickable.cycles_left / inst.components.pickable.max_cycles or 1
    return (percent >= .9 and "3")
        or (percent >= .33 and "2")
        or "1"
end

local function onpickedfn(inst, picker)
    playanim( inst, "0" )
end

local function makefullfn(inst)
    playanim( inst, pickanim( inst ) )
end

local function onsave(inst, data)
    data.build = inst.animbuild
end

local function onload(inst, data)
    if data ~= nil and data.build ~= nil then
        inst.animbuild = data.build
    end
    
    playanim(inst, pickanim( inst ))   
end

local function onhammered(inst)
    local fx = SpawnPrefab("collapse_small")
    fx.Transform:SetPosition(inst.Transform:GetWorldPosition())
    fx:SetMaterial("pot")
    inst.components.lootdropper:DropLoot()
    inst:Remove()
end

local function MakePottedPlant(name, bank, build)

    local function fn()
        local inst = CreateEntity()

        inst.entity:AddTransform()
        inst.entity:AddSoundEmitter()
        inst.entity:AddAnimState()
        inst.entity:AddNetwork()

        inst.AnimState:SetBank(bank)
        inst.AnimState:SetBuild(build)
        
        MakeSnowCoveredPristine(inst)
        
        inst.Transform:SetScale(1.5, 1.5, 1.5)
        
        inst.entity:SetPristine()

        if not TheWorld.ismastersim then
            return inst
        end        

        inst.animbuild = "bush0" .. tostring(math.random( 2 ) .."-" )
        inst.AnimState:OverrideSymbol("ferns_potted01", "berry", inst.animbuild .. "3")
        if name == "pottedplant_berries_gray" then
            inst.AnimState:OverrideSymbol("pot", "pot_gray", "pot")
        end
        inst.AnimState:PlayAnimation("f1")
        
        inst:AddComponent("inspectable")

        MakeSmallBurnable(inst)
        MakeSmallPropagator(inst)
        MakeHauntableWork(inst)

        inst:AddComponent("workable")
        inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
        inst.components.workable:SetWorkLeft(1)
        inst.components.workable:SetOnFinishCallback(onhammered)
        
        inst:AddComponent("pickable")
        inst.components.pickable.picksound = "dontstarve/wilson/harvest_berries"
        if name == "pottedplant_berries_gray" then
            inst.components.pickable:SetUp("berries", TUNING.BERRY_REGROW_TIME * 6)
        else
            inst.components.pickable:SetUp("berries", TUNING.BERRY_REGROW_TIME * 3)
        end
        inst.components.pickable.onpickedfn = onpickedfn
        inst.components.pickable.makeemptyfn = makeemptyfn
        inst.components.pickable.makebarrenfn = makebarrenfn
        inst.components.pickable.makefullfn = makefullfn
        inst.components.pickable.max_cycles = (TUNING.BERRYBUSH_CYCLES / 3) + 1
        inst.components.pickable.cycles_left = inst.components.pickable.max_cycles
                
        playanim(inst, pickanim( inst ))
        
        inst:AddComponent("lootdropper")
        
        inst:AddComponent("witherable")
        
        MakeSnowCovered(inst)
        MakeNoGrowInWinter(inst)
        
        --------SaveLoad
        inst.OnSave = onsave 
        inst.OnLoad = onload 
        inst.OnBuiltFn = function(inst)
            inst.components.pickable:MakeEmpty()
        end
        inst.OnEntityWake = OnEntityWake

        return inst
    end

    local function fn_placer()
        local inst = CreateEntity()

        --[[Non-networked entity]]

        inst.entity:AddTransform()
        inst.entity:AddAnimState()
        inst.AnimState:SetBank(bank)
        inst.AnimState:SetBuild(build)
        inst.AnimState:SetLightOverride(1)
        
        inst.AnimState:OverrideSymbol("ferns_potted01", "berry", "bush0" .. tostring(math.random( 2 ) .. "-3" ) )
        inst.AnimState:PlayAnimation("f1")

        inst:AddComponent("placer")
        inst.persists = false

        return inst
    end
    
    return Prefab(name, fn, assets, prefabs),
        Prefab(name .. "_placer", fn_placer)
end

local _prefabs = {}
local prefab, placer = MakePottedPlant("pottedplant_berries", "ferns_potted", "cave_ferns_potted")
table.insert(_prefabs, prefab)
table.insert(_prefabs, placer)

local prefab, placer = MakePottedPlant("pottedplant_berries_gray", "ferns_potted", "cave_ferns_potted")
table.insert(_prefabs, prefab)
table.insert(_prefabs, placer)

return unpack( _prefabs )