local assets =
{
    Asset("ANIM", "anim/cave_ferns_potted.zip"),
    Asset("ANIM", "anim/cacti.zip"),
}

local prefabs =
{
    "collapse_small",
}

local function onpickedfn(inst, picker)
    inst.AnimState:OverrideSymbol("ferns_potted01", "", "")
    inst.animbuild = ""
    if picker.components.combat and not picker:HasTag("thorny_protection") then
        picker.components.combat:GetAttacked(inst, TUNING.cacti_DAMAGE)
        picker:PushEvent("thorns")
    end

    if inst.has_flower then -- You get a cacti flower, yay.
        if picker ~= nil and picker.components.inventory ~= nil then
            local loot = SpawnPrefab("cactus_flower")
            if loot ~= nil then
                if loot.components.inventoryitem ~= nil then
                    loot.components.inventoryitem:InheritMoisture(TheWorld.state.wetness, TheWorld.state.iswet)
                end
                picker.components.inventory:GiveItem(loot, nil, Vector3(TheSim:GetScreenPos(inst.Transform:GetWorldPosition())))
            end
        end
    end
    inst.has_flower = false
    
    if inst:HasTag("flower") then
        inst:RemoveTag("flower")
    end
end

local function onregenfn(inst)
    if TheWorld.state.issummer then
        inst.animbuild = 1
        inst.AnimState:OverrideSymbol("ferns_potted01", "cacti", "cacti01-" .. inst.animbuild)
        inst.has_flower = true
        inst:AddTag("flower")
    else
        inst.animbuild = 0
        inst.AnimState:OverrideSymbol("ferns_potted01", "cacti", "cacti01-" .. inst.animbuild)
        inst.has_flower = false
    end
end

local function makeemptyfn(inst)
    inst.AnimState:OverrideSymbol("ferns_potted01", "", "")
    inst.has_flower = false
end

local function OnEntityWake(inst)
    if TheWorld.state.issummer  then
        if inst.components.pickable and inst.components.pickable.canbepicked then
            inst.animbuild = 1
            inst.AnimState:OverrideSymbol("ferns_potted01", "cacti", "cacti01-" .. inst.animbuild)
            inst.has_flower = true
            inst:AddTag("flower")
        else
            inst.AnimState:OverrideSymbol("ferns_potted01", "", "")
            inst.has_flower = false
        end
    else
        if inst.components.pickable and inst.components.pickable.canbepicked then
            inst.animbuild = 0
            inst.AnimState:OverrideSymbol("ferns_potted01", "cacti", "cacti01-" .. inst.animbuild)
        else
            inst.AnimState:OverrideSymbol("ferns_potted01", "", "")
        end
        inst.has_flower = false
    end
end

local function onsave(inst, data)
    data.anim = inst.animbuild
end

local function onload(inst, data)
    if data ~= nil and data.anim ~= nil then
        inst.animbuild = data.anim
        inst.AnimState:OverrideSymbol("ferns_potted01", "cacti", "cacti01-" .. inst.animbuild)
    end    
end

local function onhammered(inst)
    local fx = SpawnPrefab("collapse_small")
    fx.Transform:SetPosition(inst.Transform:GetWorldPosition())
    fx:SetMaterial("pot")
    inst.components.lootdropper:DropLoot()
    inst:Remove()
end

local function DieInDarkness(inst)
    local x,y,z = inst.Transform:GetWorldPosition()
    local ents = TheSim:FindEntities(x,0,z, DAYLIGHT_SEARCH_RANGE, {"daylight"})
    local wither = true
    for k,v in pairs(ents) do
        if v.Light then
            local darkness_sq = v.Light:GetCalculatedRadius() * 0.7
            darkness_sq = darkness_sq * darkness_sq
            if inst:GetDistanceSqToInst(v) < darkness_sq then
                wither = false
                break
            end
        end
    end
    if wither then
        inst.AnimState:OverrideSymbol("ferns_potted01", "", "")    
        inst.animbuild = ""    
        inst.components.pickable:MakeEmpty()
    end
end

local function OnIsCaveDay(inst, isday)
    if isday then
        inst:DoTaskInTime(5.0 + math.random()*5.0, DieInDarkness)
    end
end

local function MakePottedPlant(name, bank, build)

    local function fn()
        local inst = CreateEntity()

        inst.entity:AddTransform()
        inst.entity:AddSoundEmitter()
        inst.entity:AddAnimState()
        inst.entity:AddNetwork()

        inst.AnimState:SetBank(bank)
        inst.AnimState:SetBuild(build)
        inst.AnimState:SetRayTestOnBB(true)

        inst:AddTag("thorny")
        
        inst.entity:SetPristine()

        if not TheWorld.ismastersim then
            return inst
        end        

        inst.animbuild = 0
        if TheWorld.state.issummer then
            inst.animbuild = 1
        else
            inst.animbuild = 0
        end
        inst.AnimState:OverrideSymbol("ferns_potted01", "cacti", "cacti01-" .. inst.animbuild)
        if name == "pottedplant_cactus_gray" then
            inst.AnimState:OverrideSymbol("pot", "pot_gray", "pot")
        end
        inst.AnimState:PlayAnimation("f1")
        inst:AddComponent("inspectable")

        MakeSmallBurnable(inst)
        MakeSmallPropagator(inst)
        MakeHauntableWork(inst)

        inst:AddComponent("workable")
        inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
        inst.components.workable:SetWorkLeft(1)
        inst.components.workable:SetOnFinishCallback(onhammered)
        
        inst:AddComponent("pickable")
        inst.components.pickable.picksound = "dontstarve/wilson/pickup_plants"
        if name == "pottedplant_cactus_gray" then
            inst.components.pickable:SetUp("cactus_meat", TUNING.CACTUS_REGROW_TIME * 6)
        else
            inst.components.pickable:SetUp("cactus_meat", TUNING.CACTUS_REGROW_TIME * 3)
        end
        inst.components.pickable.onregenfn = onregenfn
        inst.components.pickable.onpickedfn = onpickedfn
        inst.components.pickable.makeemptyfn = makeemptyfn

        inst:AddComponent("lootdropper")

        if TheWorld:HasTag("cave") then
            inst:WatchWorldState("iscaveday", OnIsCaveDay)
        end

        
        --------SaveLoad
        inst.OnSave = onsave 
        inst.OnLoad = onload 
        inst.OnEntityWake = OnEntityWake

        return inst
    end

    local function fn_placer()
        local inst = CreateEntity()

        --[[Non-networked entity]]

        inst.entity:AddTransform()
        inst.entity:AddAnimState()
        inst.AnimState:SetBank(bank)
        inst.AnimState:SetBuild(build)
        inst.AnimState:SetLightOverride(1)
        
        inst.animbuild = 0
        if TheWorld.state.issummer then
            inst.animbuild = 1
        else
            inst.animbuild = 0
        end
        inst.AnimState:OverrideSymbol("ferns_potted01", "cacti", "cacti01-" .. inst.animbuild)
        inst.AnimState:PlayAnimation("f1")
        
        inst:AddComponent("placer")
        inst.persists = false

        return inst
    end
    
    return Prefab(name, fn, assets, prefabs),
        Prefab(name .. "_placer", fn_placer)
end

local _prefabs = {}
local prefab, placer = MakePottedPlant("pottedplant_cactus", "ferns_potted", "cave_ferns_potted")
table.insert(_prefabs, prefab)
table.insert(_prefabs, placer)

local prefab, placer = MakePottedPlant("pottedplant_cactus_gray", "ferns_potted", "cave_ferns_potted")
table.insert(_prefabs, prefab)
table.insert(_prefabs, placer)

return unpack( _prefabs )