local assets =
{
    Asset("ANIM", "anim/cave_ferns_potted.zip"),
}

local prefabs =
{
    "collapse_small",
}

local function playsymbol(inst)
    inst.animname = "f" .. tostring(math.random(10))
    while (inst.animname == "f8") do
        inst.animname = "f" .. tostring(math.random(10))
    end        
    inst.AnimState:OverrideSymbol("ferns_potted01", inst.overridesymbol, inst.overridesymbol .. "01")
    inst.AnimState:PlayAnimation(inst.animname)
end

local function onsave(inst, data)
    data.anim = inst.animname
    data.overridesymbol = inst.overridesymbol
end

local function onload(inst, data)
    if data ~= nil and data.anim ~= nil then
        inst.animname = data.anim
    end
    
    if data ~= nil and data.overridesymbol ~= nil then
        inst.overridesymbol = data.overridesymbol
    end
    
    if inst.components.pickable:CanBePicked() then
        playsymbol(inst)
    end
end

local function onpickedfn(inst, picker)
    if picker and picker.components.sanity and inst.overridesymbol == "flowers" then
        picker.components.sanity:DoDelta(TUNING.SANITY_TINY)
    else
        picker.components.sanity:DoDelta(-TUNING.SANITY_TINY)
    end
    
    inst.AnimState:OverrideSymbol("ferns_potted01", "", "")
end

local function makeemptyfn(inst)
    inst.AnimState:OverrideSymbol("ferns_potted01", "", "")
    inst:RemoveTag("flower")
end

local function onhammered(inst)
    local fx = SpawnPrefab("collapse_small")
    fx.Transform:SetPosition(inst.Transform:GetWorldPosition())
    fx:SetMaterial("pot")
    inst.components.lootdropper:DropLoot()
    inst:Remove()
end

local function DieInDarkness(inst)
    local x,y,z = inst.Transform:GetWorldPosition()
    local ents = TheSim:FindEntities(x,0,z, DAYLIGHT_SEARCH_RANGE, {"daylight"})
    local wither = true
    for k,v in pairs(ents) do
        if v.Light then
            local darkness_sq = v.Light:GetCalculatedRadius() * 0.7
            darkness_sq = darkness_sq * darkness_sq
            if inst:GetDistanceSqToInst(v) < darkness_sq then
                wither = false
                break
            end
        end
    end
    if wither then
        inst.AnimState:OverrideSymbol("ferns_potted01", "withered_flowers", "withered_flowers")
        inst.AnimState:PlayAnimation("f" .. tostring( math.random(3) ) )
        
        inst.components.pickable.product = "grass"
    end
end

local function OnIsCaveDay(inst, isday)
    if isday then
        inst:DoTaskInTime(5.0 + math.random()*5.0, DieInDarkness)
    end
end


local function makefullfn(inst)
     playsymbol(inst)
   
    --if inst.overridesymbol == "flowers" then
        inst.components.pickable.product = "petals"
    --else
    --    inst.components.pickable.product = "petals_evil"
    --end
    
    inst:AddTag("flower")
end

local function MakePottedPlant(name, bank, build)

    local function overridesymbol()
        --local overridesymbol = ""
        --if math.random(1, 1000) > 500 then
            return "flowers"
        --else
        --   return "flowers_evil"
        --end
    end

    local function fn()
        local inst = CreateEntity()

        inst.entity:AddTransform()
        inst.entity:AddSoundEmitter()
        inst.entity:AddAnimState()
        inst.entity:AddNetwork()

        inst.AnimState:SetBank(bank)
        inst.AnimState:SetBuild(build)
        inst.AnimState:SetRayTestOnBB(true)

        inst:AddTag("flower")
        
        inst.entity:SetPristine()

        if not TheWorld.ismastersim then
            return inst
        end        

        inst.overridesymbol = overridesymbol()
        playsymbol(inst)

        inst:AddComponent("inspectable")

        MakeSmallBurnable(inst)
        MakeSmallPropagator(inst)
        MakeHauntableWork(inst)

        inst:AddComponent("workable")
        inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
        inst.components.workable:SetWorkLeft(1)
        inst.components.workable:SetOnFinishCallback(onhammered)
        
        inst:AddComponent("pickable")
        inst.components.pickable.picksound = "dontstarve/wilson/pickup_plants"
        --if inst.overridesymbol == "flowers" then
            inst.components.pickable:SetUp("petals", (TUNING.FLOWER_REGROWTH_TIME * TUNING.TOTAL_DAY_TIME) / 10)
        --else
        --    inst.components.pickable:SetUp("petals_evil", (TUNING.FLOWER_REGROWTH_TIME * TUNING.TOTAL_DAY_TIME) / 10)
        --end
        inst.components.pickable.onpickedfn = onpickedfn
        inst.components.pickable.quickpick = true
        inst.components.pickable.wildfirestarter = false
        inst.components.pickable.makeemptyfn = makeemptyfn
        inst.components.pickable.makefullfn = makefullfn

        inst:AddComponent("lootdropper")
        
        if TheWorld:HasTag("cave") then
            inst:WatchWorldState("iscaveday", OnIsCaveDay)
        end


        --------SaveLoad
        inst.OnSave = onsave 
        inst.OnLoad = onload 

        return inst
    end

    local function fn_placer()
        local inst = CreateEntity()

        --[[Non-networked entity]]

        inst.entity:AddTransform()
        inst.entity:AddAnimState()
        inst.AnimState:SetBank(bank)
        inst.AnimState:SetBuild(build)
        inst.AnimState:SetLightOverride(1)
        
        inst.overridesymbol = overridesymbol()
        playsymbol(inst)

        inst:AddComponent("placer")
        inst.persists = false

        return inst
    end
    
    return Prefab(name, fn, assets, prefabs),
        Prefab(name .. "_placer", fn_placer)
end

local _prefabs = {}
local prefab, placer = MakePottedPlant("pottedplant_flower", "ferns_potted", "cave_ferns_potted")
table.insert(_prefabs, prefab)
table.insert(_prefabs, placer)

return unpack( _prefabs )


